@@ -7,7 +7,7 @@ _get_ves()
 	local cmd=$1
 	local rm_empty='sed s/^-[[:space:]]*//'
 	# Only allow this completion when user has root privileges
-	[ "$EUID" -ne 0 ] && exit 1
+	[ "$EUID" -ne 0 ] && return 1
 	case $cmd in
 		create)
 			# create a new CTID, by increasing the last one
@@ -40,7 +40,7 @@ _get_ves()
 				grep '^simfs ' | \
 				sed 's/^simfs *//' | $rm_empty
 			;;
-		snapshot*)
+		compact|snapshot*)
 			# CTs on ploop
 			(/usr/sbin/vzlist -H -a -o layout,ctid;
 			 /usr/sbin/vzlist -H -a -o layout,name) 2>/dev/null | \
@@ -73,7 +73,8 @@ _vzctl()
 		set start stop restart status quotaon quotaoff quotainit \
 		enter exec exec2 runscript console"
 	local vzctl_create_opts="--ostemplate --config --private --root \
-		--layout --diskspace --ipadd --hostname --name --description"
+		--layout --diskspace --diskinodes --ipadd --hostname \
+		--name --description"
 	local vzctl_convert_opts="--layout"
 	local vzctl_set_opts="--save --onboot --root --private \
 		--userpasswd --disabled --ipadd --ipdel \
@@ -84,27 +85,24 @@ _vzctl()
 		--shmpages --numfile --numflock --numpty --numsiginfo \
 		--dcachesize --numiptent --physpages --swappages \
 		--vm_overcommit --cpuunits --cpulimit --cpus --cpumask \
-		--iptables --netdev_add --netdev_del --netif_add --netif_del \
+		--netfilter --netdev_add --netdev_del --netif_add --netif_del \
 		--diskquota --mount_opts \
 		--diskspace --diskinodes --quotatime --quotaugidlimit \
 		--capability --devnodes --devices --applyconfig \
 		--name --ioprio --iolimit --iopslimit \
 		--setmode --meminfo --bootorder --features \
-		--reset_ub --stop-timeout"
+		--reset_ub --stop-timeout --offline-resize"
 	local vzctl_start_opts="--wait --force --skip-fsck"
 	local vzctl_stop_opts="--fast --skip-umount"
 	local vzctl_restart_opts="$vzctl_start_opts $vzctl_stop_opts"
 	local vzctl_snapshot_create_opts="--id --name --description \
 		--skip-suspend --skip-config"
+	local vzctl_snapshot_switch_opts="--id --skip-resume --skip-config"
 	local vzctl_snapshot_mount_opts="--id --target"
 	local vzctl_snapshot_list_opts="-H --no-header -o --output --id \
 		-L --list --help"
 
-	local iptables_names="iptable_filter iptable_mangle ipt_limit
-		ipt_multiport ipt_tos ipt_TOS ipt_REJECT ipt_TCPMSS
-		ipt_tcpmss ipt_ttl ipt_LOG ipt_length ip_conntrack
-		ip_conntrack_ftp ip_conntrack_irc ipt_conntrack ipt_state
-		ipt_helper iptable_nat ip_nat_ftp ip_nat_irc ipt_REDIRECT"
+	local netfilter_names="disabled stateless stateful full"
 
 	local cap_names="chown dac_override dac_read_search fowner fsetid kill
 		setgid setuid setpcap linux_immutable net_bind_service
@@ -169,8 +167,8 @@ _vzctl()
 			--layout)
 				COMPREPLY=( $( compgen -W "ploop ploop:expanded ploop:plain ploop:raw simfs" -- $cur ) )
 				;;
-			--iptables)
-				COMPREPLY=( $( compgen -W "$iptables_names" -- $cur ) )
+			--netfilter)
+				COMPREPLY=( $( compgen -W "$netfilter_names" -- $cur ) )
 				;;
 			--netdev*)
 				local devs=`ip addr show | awk '/^[0-9]/ && /UP/ && !/venet/ && !/lo/ \
@@ -220,7 +218,7 @@ _vzctl()
 			--ipdel)
 				# Only allow this completion
 				# when user has root privileges
-				[ "$EUID" -ne 0 ] && exit 1
+				[ "$EUID" -ne 0 ] && return 1
 				# Get CTID
 				local cmd=${COMP_WORDS[1]}
 				local ve=${COMP_WORDS[2]}
@@ -260,7 +258,7 @@ _vzctl()
 			--id|--uuid)
 				# Only allow this completion
 				# when user has root privileges
-				[ "$EUID" -ne 0 ] && exit 1
+				[ "$EUID" -ne 0 ] && return 1
 				# Get command and CTID
 				local cmd=${COMP_WORDS[1]}
 				local ve=${COMP_WORDS[2]}
@@ -273,17 +271,17 @@ _vzctl()
 				case ${cmd} in
 				    snapshot-umount)
 					# For umount, only give mounted IDs
-					ids="`/usr/sbin/vzctl snapshot-list $ve -H -ouuid,device 2>/dev/null | awk 'NF==2 {print $1}' | tr -d '{}'`"
+					ids="`/usr/sbin/vzctl --quiet snapshot-list $ve -H -ouuid,device 2>/dev/null | awk 'NF==2 {print $1}' | tr -d '{}'`"
 					;;
 				    snapshot-*)
 					# For other commands, give all IDs
-					ids="`/usr/sbin/vzctl snapshot-list $ve -H -ouuid 2>/dev/null | tr -d '{}'`"
+					ids="`/usr/sbin/vzctl --quiet snapshot-list $ve -H -ouuid 2>/dev/null | tr -d '{}'`"
 					;;
 				    *)
 					# Do not give anything for other cmds
 					# including 'snapshot' itself
 					# since it requires a new unused ID
-					exit 0
+					return 0
 					;;
 				esac
 				COMPREPLY=( $( compgen -W "$ids" -- $cur ) )
@@ -322,7 +320,10 @@ _vzctl()
 					snapshot)
 						COMPREPLY=( $( compgen -W "$vzctl_snapshot_create_opts" -- $cur ) )
 						;;
-					snapshot-switch|snapshot-delete|snapshot-umount)
+					snapshot-switch)
+						COMPREPLY=( $( compgen -W "$vzctl_snapshot_switch_opts" -- $cur ) )
+						;;
+					snapshot-delete|snapshot-umount)
 						COMPREPLY=( $( compgen -W "--id" -- $cur ) )
 						;;
 					snapshot-mount)
